@@ -24,8 +24,8 @@ from typing_extensions import TypeAlias
 
 from mpcrl.core.callbacks import AgentCallbacks, RemovesCallbackHooksInState
 from mpcrl.core.exploration import ExplorationStrategy, NoExploration
+from mpcrl.util.iters import generate_seeds
 from mpcrl.util.named import Named
-from mpcrl.util.random import generate_seeds
 
 SymType = TypeVar("SymType", cs.SX, cs.MX)
 ActType: TypeAlias = Union[npt.ArrayLike, Dict[str, npt.ArrayLike]]
@@ -163,6 +163,7 @@ class Agent(
         vals0: Union[
             None, Dict[str, npt.ArrayLike], Iterable[Dict[str, npt.ArrayLike]]
         ] = None,
+        store_solution: bool = True,
     ) -> Solution[SymType]:
         """Solves the agent's specific MPC optimal control problem.
 
@@ -185,7 +186,11 @@ class Agent(
             keys are the names of the MPC variables, and values are the numerical
             initial values of each variable. Use this to warm-start the MPC. If `None`,
             and a previous solution (possibly, successful) is available, the MPC solver
-            is automatically warm-started
+            is automatically warm-started.
+        store_solution : bool, optional
+            By default, the mpc solution is stored accordingly to the `warmstart`
+            strategy. If set to `False`, this flag allows to disable the behaviour for
+            this particular solution.
 
         Returns
         -------
@@ -229,8 +234,8 @@ class Agent(
             vals0 = self._last_solution.vals
 
         # solve and store solution
-        sol = mpc(pars=pars, vals0=vals0)
-        if not self._store_last_successful or sol.success:
+        sol = mpc(pars, vals0)
+        if store_solution and (not self._store_last_successful or sol.success):
             self._last_solution = sol
         return sol
 
@@ -241,6 +246,7 @@ class Agent(
         vals0: Union[
             None, Dict[str, npt.ArrayLike], Iterable[Dict[str, npt.ArrayLike]]
         ] = None,
+        **kwargs,
     ) -> Tuple[cs.DM, Solution[SymType]]:
         """Computes the state value function `V(s)` approximated by the MPC.
 
@@ -259,7 +265,7 @@ class Agent(
             keys are the names of the MPC variables, and values are the numerical
             initial values of each variable. Use this to warm-start the MPC. If `None`,
             and a previous solution (possibly, successful) is available, the MPC solver
-            is automatically warm-started
+            is automatically warm-started.
 
         Returns
         -------
@@ -269,13 +275,13 @@ class Agent(
             The solution of the MPC approximating `V(s)` at the given state.
         """
         if deterministic or not self._exploration.can_explore():
-            perturbation = None
+            pert = None
         else:
-            perturbation = self._exploration.perturbation(
+            pert = self._exploration.perturbation(
                 self.cost_perturbation_method,
                 size=self.V.parameters[self.cost_perturbation_parameter].shape,
             )
-        sol = self.solve_mpc(self._V, state, perturbation=perturbation, vals0=vals0)
+        sol = self.solve_mpc(self._V, state, perturbation=pert, vals0=vals0, **kwargs)
         first_action = cs.vertcat(*(sol.vals[u][:, 0] for u in self._V.actions.keys()))
         return first_action, sol
 
@@ -286,6 +292,7 @@ class Agent(
         vals0: Union[
             None, Dict[str, npt.ArrayLike], Iterable[Dict[str, npt.ArrayLike]]
         ] = None,
+        **kwargs,
     ) -> Solution[SymType]:
         """Computes the action value function `Q(s,a)` approximated by the MPC.
 
@@ -311,7 +318,7 @@ class Agent(
         Solution
             The solution of the MPC approximating `Q(s,a)` at given state and action.
         """
-        return self.solve_mpc(self._Q, state, action=action, vals0=vals0)
+        return self.solve_mpc(self._Q, state, action, vals0=vals0, **kwargs)
 
     def evaluate(
         self,
@@ -358,7 +365,7 @@ class Agent(
         self.on_validation_start(env)
 
         for episode, current_seed in zip(range(episodes), generate_seeds(seed)):
-            self.reset(seed=current_seed)
+            self.reset(current_seed)
             state, _ = env.reset(seed=current_seed, options=env_reset_options)
             truncated, terminated, timestep = False, False, 0
             self.on_episode_start(env, episode)
@@ -369,10 +376,11 @@ class Agent(
                     self.on_mpc_failure(episode, timestep, sol.status, raises)
 
                 state, r, truncated, terminated, _ = env.step(action)
+                self.on_env_step(env, episode, timestep)
 
                 returns[episode] += r
                 timestep += 1
-                self.on_env_step(env, episode, timestep)
+                self.on_timestep_end(env, episode, timestep)
 
             self.on_episode_end(env, episode, returns[episode])
 
@@ -392,7 +400,7 @@ class Agent(
         a0 = Q.nlp.parameter(self.init_action_parameter, (na, 1))
         perturbation = V.nlp.parameter(self.cost_perturbation_parameter, (na, 1))
 
-        u0 = cs.vertcat(*mpc.first_actions.values())
+        u0 = cs.vcat(mpc.first_actions.values())
         f = V.nlp.f
         if mpc.is_wrapped(wrappers.NlpScaling):
             f = mpc.scale(f)
