@@ -1,9 +1,11 @@
-from itertools import chain, repeat
+from itertools import repeat
 from typing import (
     Any,
+    Callable,
     Collection,
     Dict,
     Generic,
+    Iterator,
     List,
     Literal,
     Optional,
@@ -218,7 +220,7 @@ class LstdDpgAgent(RlLearningAgent[SymType, ExpType, LrType], Generic[SymType, L
             name,
         )
         # initialize derivatives, state feature vector and hessian approximation
-        self._sensitivity = self._init_sensitivities(linsolver)
+        self._sensitivity = self._init_sensitivity(linsolver)
         self._Phi = (
             monomials_basis_function(mpc.ns, 0, 2)
             if state_features is None
@@ -249,22 +251,9 @@ class LstdDpgAgent(RlLearningAgent[SymType, ExpType, LrType], Generic[SymType, L
         )
 
     def update(self) -> Optional[str]:
+        # sample and congregate sampled rollouts into a unique least-squares problem
         sample = self.experience.sample()
-
-        # congregate sample's rollout into a unique least-squares problem. Since the
-        # sample is usually shortish, no point in using nb.jit
-        L_, Phi_, Psi_, dpidtheta_ = [], [], [], []
-        mask_phi = [False]
-        for L, Phi, Psi, dpidtheta in sample:
-            L_.append(L)
-            Phi_.append(Phi)
-            Psi_.append(Psi)
-            mask_phi.extend(chain(repeat(True, L.shape[0]), (False,)))
-            dpidtheta_.append(dpidtheta)
-        L = np.concatenate(L_)
-        Phi = np.concatenate(Phi_)
-        Psi = np.concatenate(Psi_)
-        dpidtheta = np.concatenate(dpidtheta_)
+        L, Phi, Psi, dpidtheta, mask_phi = _congregate_rollouts_sample(sample)
 
         # compute CAFA weights v
         Phi_diff = self.discount_factor * Phi[mask_phi[:-1]] - Phi[mask_phi[1:]]
@@ -345,12 +334,14 @@ class LstdDpgAgent(RlLearningAgent[SymType, ExpType, LrType], Generic[SymType, L
             self._consolidate_rollout_into_memory()
         return rewards
 
-    def _init_sensitivities(self, linsolvertype: str) -> cs.Function:
+    def _init_sensitivity(
+        self, linsolver_type: str
+    ) -> Callable[[cs.DM, int], np.ndarray]:
         """Internal utility to compute the derivatives w.r.t. the learnable parameters
         and other functions in order to estimate the policy gradient."""
         nlp = self._V.nlp
         y = nlp.primal_dual
-        theta = cs.vcat(self._learnable_pars.sym.values())
+        theta = cs.vvcat(self._learnable_pars.sym.values())
         u0 = cs.vcat(self._V.first_actions.values())
         x_lam_p = cs.vertcat(nlp.primal_dual, nlp.p)
 
@@ -360,26 +351,27 @@ class LstdDpgAgent(RlLearningAgent[SymType, ExpType, LrType], Generic[SymType, L
         Ky = nlp_.jacobians["K-y"].T
         dydu0 = cs.evalf(cs.jacobian(u0, y)).T
 
-        # convert SX to MX (so that we can use the linsolver)
+        # instantiate linear solver (must be MX)
         if nlp.sym_type is cs.SX:
+            # convert SX to MX (so that we can use the linsolver)
             x_lam_p, x_lam_p_sx = cs.MX.sym("in", *x_lam_p.shape), x_lam_p
-            Kt = cs.Function("dKdtheta", [x_lam_p_sx], [Kt]).wrap()(x_lam_p)
-            Ky = cs.Function("dKdy", [x_lam_p_sx], [Ky]).wrap()(x_lam_p)
+            Kt, Ky = cs.Function("sx2mx", (x_lam_p_sx,), (Kt, Ky))(x_lam_p)
+        linsolver = (
+            cs.solve  # cs.mldivide
+            if linsolver_type == "mldivide"
+            else cs.Linsol("linsolver", linsolver_type, Ky.sparsity()).solve
+        )
 
-        # compute last bunch derivative and convert to function for faster runtime
-        if linsolvertype == "mldivide":
-            q = cs.solve(Ky, dydu0)  # cs.mldivide
-        else:
-            linsolver = cs.Linsol("linsolver", linsolvertype, Ky.sparsity())
-            q = linsolver.solve(Ky, dydu0)
-        dpidtheta = -Kt @ q
-        sensitivity = cs.Function("dpidtheta", (x_lam_p,), (dpidtheta,))
-
-        # wrap to conveniently return arrays. Casadi does not support tensors with more
-        # than 2 dims, so dpidtheta gets squished in the 3rd dim and needs reshaping
+        # compute sensitivity and convert to function (faster runtime)
+        dpidtheta = -Kt @ linsolver(Ky, dydu0)
+        sensitivity = cs.Function(
+            "pi_sensitivity", (x_lam_p,), (dpidtheta,), ("x_lam_p",), ("dpidtheta",)
+        )
         ntheta, na = dpidtheta.shape
 
         def func(sol_values: cs.DM, N: int) -> np.ndarray:
+            # wrap to conveniently return arrays. Casadi does not support tensors with
+            # >2 dims, so dpidtheta gets squished in the 3rd dim and needs reshaping
             return (
                 sensitivity(sol_values.T)
                 .full()
@@ -428,3 +420,23 @@ def _consolidate_rollout(
         L[i] = cost
         sol_vals[i] = sol_val.reshape(-1)
     return N, S, E, L, sol_vals
+
+
+def _congregate_rollouts_sample(
+    sample: Iterator[ExpType],
+) -> Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray, List[bool]]:
+    """Internal utility to congregate a sample of rollouts into single arrays."""
+    # jit does not seem to provide any speedup here
+    L_, Phi_, Psi_, dpidtheta_, mask_phi = [], [], [], [], [False]
+    for L, Phi, Psi, dpidtheta in sample:
+        L_.append(L)
+        Phi_.append(Phi)
+        Psi_.append(Psi)
+        dpidtheta_.append(dpidtheta)
+        mask_phi.extend(repeat(True, L.shape[0]))
+        mask_phi.append(False)
+    L = np.concatenate(L_)
+    Phi = np.concatenate(Phi_)
+    Psi = np.concatenate(Psi_)
+    dpidtheta = np.concatenate(dpidtheta_)
+    return L, Phi, Psi, dpidtheta, mask_phi
